/* Osciloscopio con Arduino  183.000 m/s
 * realizado SETA43
   xsetaseta@gmail.com
   seta43.duckdns.org
   http://seta43.blogspot.com/
 * basado en:
 * 			Oscilloscope library for Processing ArduinoScope
 *			2009 David Konsumer <david.konsumer@gmail.com>
 * 05/04/2020
 * Usar con Arduino y programa osci_java.ino
 */


import processing.core.*;
import processing.xml.*;

import arduinoscope.*;
import processing.serial.*;
import controlP5.*;

import java.applet.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;
import java.util.zip.*;
import java.util.regex.*;

public class SimpleSerialArduinoscope extends PApplet {
	Oscilloscope scopes;

	Serial port;
	ControlP5 controlP5;

	PFont fontLarge, fontSmall;

	int LINE_FEED = 10;

	int[] vals;

	public static final int ANCHO = 800;
	public static final int ALTO = 512;

	int[] valores = new int[ANCHO];

	int contador;

	int otra = 1, pausa = 0, invertir = 0, vreferencia = 0, zoom = 1, grill = 0;
	int voltioMax, voltioMin, voltioRef = 180;

	public void setup() {
		int vals;

		 size(ANCHO, ALTO, P2D);
		 background(0);

		 controlP5 = new ControlP5(this);

		 fontLarge = loadFont("TrebuchetMS-20.vlw");
		 fontSmall = loadFont("Uni0554-8.vlw");

		 print("CANALES ");
		 println(1);
		 print("W ");
		 println(width);
		 print("H ");
		 println(height);

		int[] dimv = new int[2];
		int[] posv = new int[2];

		 posv[0] = 0;
		 posv[1] = 0;
		 dimv[0] = width;
		 dimv[1] = height;
		 scopes = new Oscilloscope(this, posv, dimv);

		 controlP5.addButton("PAUSA", 1, 720, 20, 50, 20).setId(10);
		 controlP5.addButton("START", 1, 720, 40, 50, 20).setId(11);

		 controlP5.addButton("INVERT", 1, 720, 80, 50, 20).setId(12);

		 controlP5.addButton("EXIT", 1, 720, 480, 50, 20).setId(100);

		 controlP5.addButton("DIV4", 1, 720, 240, 50, 20).setId(21);
		 controlP5.addButton("DIV8", 1, 720, 260, 50, 20).setId(22);
		 controlP5.addButton("DIV16", 1, 720, 280, 50, 20).setId(23);
		 controlP5.addButton("DIV32", 1, 720, 300, 50, 20).setId(24);
		 controlP5.addButton("DIV64", 1, 720, 320, 50, 20).setId(25);
		 controlP5.addButton("DIV128", 1, 720, 340, 50, 20).setId(26);

		 controlP5.addButton("Vref5V", 1, 720, 380, 50, 20).setId(27);
		 controlP5.addButton("Vref1V1", 1, 720, 400, 50, 20).setId(28);

		 controlP5.addButton("RESET_REF", 1, 720, 120, 50, 20).setId(29);
		 controlP5.addButton("ZOOM", 1, 720, 160, 50, 20).setId(31);
		 controlP5.addButton("GRILL ", 1, 720, 180, 50, 20).setId(32);

		 scopes.setLine_color(color(30, 255, 0));

		 port = new Serial(this, Serial.list()[0], 115200);

// Limpia el buffer serie hasta la llegada de un retorno de carro
		 port.clear();
		 port.bufferUntil(LINE_FEED);
		 scopes.addData(10);

		for (int i = 1; i < ANCHO; i++)
			 valores[i] = 1;

		 scopes.setValues(valores);
		 scopes.setPause(true);
	} 
	
	public void draw() {
		background(0);

		scopes.drawBounds();
		fill(255, 255, 0);
		textFont(fontLarge);
		if (zoom == 2)
			text("ZOOM", 100, 20);
		if (pausa == 1)
			text("PAUSA", 10, 20);
		if (invertir == 1)
			text("INVERT", 10, 42);

		if (grill == 1) {
			//scopes.setLine_color(color(100, 255, 0));
			stroke(50, 50, 0);
			for (int y = 0; y < 512; y += 10)
				line(0, y, 699, y);
			for (int x = 0; x < 700; x += 10)
				line(x, 0, x, ALTO);
			text("GRILL 10x10", 200, 20);
		}

		if (vreferencia == 0) {
			text("MAX=" + voltioMax + " mV", 570, 20);
			text("MIN=" + voltioMin + " mV", 570, 41);
		} else {
			text("MAX=" + (voltioMax * 100) / 454 + " mV", 570, 20);
			text("MIN=" + (voltioMin * 100) / 454 + " mV", 570, 41);
		}

		fill(255, 255, 255);	//left blue vertical bar
		rect(699, 0, 2, ALTO);

		textFont(fontSmall);
		if (vreferencia == 0)
			text("V.REF_5V", 720, 430);
		else
			text("V.REF_1V1", 720, 433);

		//text(String.valueOf(contador), 720, 460);
		//contador++;

		scopes.draw();
		controlP5.draw();

		if (otra == 1 && pausa == 0) {
			otra = 0;
			if (invertir == 0)
				serialWStrLN("GETUP=");
			else
				serialWStrLN("GETDOWN=");
		}
	}



	public void controlEvent(ControlEvent theEvent) {
		int id = theEvent.controller().id();
		//println(id); 
		switch (id) {
		case 10:
			if (pausa == 0) {
				otra = 0;
				pausa = 1;
			} else {
				otra = 1;
				pausa = 0;
			}

			break;
		case 11:
			serialWStrLN("GETUP=");
			break;
		case 12:
			if (invertir == 0)
				invertir = 1;
			else
				invertir = 0;
			break;

		case 21:
			serialWStrLN("DIV4");
			break;
		case 22:
			serialWStrLN("DIV8");
			break;
		case 23:
			serialWStrLN("DIV16");
			break;
		case 24:
			serialWStrLN("DIV32");
			break;
		case 25:
			serialWStrLN("DIV64");
			break;
		case 26:
			serialWStrLN("DIV128");
			break;
		case 27:
			vreferencia = 0;
			serialWStrLN("VREF0");
			break;
		case 28:
			vreferencia = 1;
			serialWStrLN("VREF1");
			break;
		case 29:
			serialWStrLN("TEST=");
			break;
		case 31:
			if (zoom == 2)
				zoom = 1;
			else
				zoom = 2;
			break;
		case 32:
			if (grill == 1)
				grill = 0;
			else
				grill = 1;
			break;
		case 100:
			System.exit(0);
			break;
		}

	}

	public void serialWStrLN(String texto) {
		byte[]byteArrray = texto.getBytes();
		for (int i = 0; i < byteArrray.length; i++)
			port.write(byteArrray[i]);
		port.write(10);
	}


	// EVENTOS SERIE
	public void serialEvent(Serial p) {
		long z1, z2;
		int xx;

		z2 = (long) voltioRef *10;
		String data = p.readStringUntil(LINE_FEED);

		if (data != null && data.length() > 1400) {
			voltioMax = 0;
			voltioMin = 1000;
			vals = PApplet.parseInt(split(data, ' '));
			for (int i = 1; i < 700; i++) {
				z1 = vals[(i / zoom)];
				z1 = (z1 * 1683) / z2;

				if (voltioMax < z1)
					voltioMax = (int) ((z1 * 5000) / 255);
				if (voltioMin > z1)
					voltioMin = (int) ((z1 * 5000) / 255);

				valores[ANCHO - i] = (int) ((z1 * 4) + 2);
				//valores[ANCHO-i]=4*vals[i];
			}
			scopes.setValues(valores);
			//println(data.length());
		} else {
			//println(data);  
			if (data.substring(0, 3).equals("FIN"))
				otra = 1;
			if (data.substring(0, 5).equals("TEST=")) {
				String[]troceado = data.split(" ");
				String tmp = troceado[1].substring(0, troceado[1].length() - 2);

				voltioRef = Integer.parseInt(tmp);
				print("ValorREF=");
				println(voltioRef);
			}

		}
	}

	static public void main(String args[]) {
		PApplet.main(new String[] {
					 "--bgcolor=#DFDFDF", "SimpleSerialArduinoscope"});
	}
}
